/*
 * OAuth Class v0.3
 *
 * Copyleft 2013, Valentin Schmidt
 * Date: 2013-05-19
 *
 * Requirements:
 * - script "sha1"
 * - netLingo xtras
 */

_global.OAuth = {
  
  oauth_version: "1.0",
  oauth_signature_method: "HMAC-SHA1",
    
  oauth_consumer_key: null,
  oauth_consumer_secret: null,
  
  oauth_request_token: null,
  oauth_request_token_secret: null,
  
  oauth_access_token: null,
  oauth_access_token_secret: null,
  
  oauth_verifier: null
};

/**
 * OAuth.set_consumer_key
 */
OAuth.set_consumer_key = function(consumer_key, consumer_secret){
  this.oauth_consumer_key = consumer_key;
  this.oauth_consumer_secret = consumer_secret;
};

/**
 * OAuth.set_access_token
 */
OAuth.set_access_token = function(access_token, access_token_secret){
  this.oauth_access_token = access_token;
  this.oauth_access_token_secret = access_token_secret;
};

/**
 * OAuth.get_request_token
 *
 * base_url: e.g. "http://www.flickr.com/services/oauth/request_token";
 */
OAuth.get_request_token = function(base_url, oauth_callback_name, callback){
  
  var key = this.oauth_consumer_secret + "&";
  var oauth_nonce = this._createNonce(32);
  var oauth_timestamp = Math.round(new Date().getTime()/1000);

  var query_str = oauth_callback_name ? "oauth_callback="+encodeURIComponent(oauth_callback_name)+"&" : "";

  query_str += "oauth_consumer_key="+this.oauth_consumer_key;
  query_str += "&oauth_nonce="+oauth_nonce;
  query_str += "&oauth_signature_method="+this.oauth_signature_method;
  query_str += "&oauth_timestamp="+oauth_timestamp;
  query_str += "&oauth_version=1.0";

  var data = "GET" + "&" + encodeURIComponent(base_url) + "&" + encodeURIComponent(query_str);
  var sig = b64_hmac_sha1(key, data);
  var url = base_url + "?" + query_str + "&oauth_signature="+encodeURIComponent(sig);

  this._httpGet(url, function(txt, err){
    if (err!="OK"){
      callback(false, "Network error ("+ err+")");
    }else{
      res = OAuth._parseQueryString(txt);
      if (res.oauth_callback_confirmed=='true'){
        OAuth.oauth_request_token = res.oauth_token;
        OAuth.oauth_request_token_secret = res.oauth_token_secret;
        callback(true);
      }else{
        callback(false, "Error getting request token!", res);
      }
    }
  });
  
};

/**
 * OAuth.authorize
 *
 * var base_url = "http://www.flickr.com/services/oauth/authorize";
 */
OAuth.authorize = function(base_url){
  gotoNetPage(base_url+"?oauth_token="+this.oauth_request_token);
};

/**
 * OAuth.set_verifier
 */
OAuth.set_verifier = function(verifier){
  this.oauth_verifier = verifier;
};

/**
 * OAuth.get_access_token
 *
 * Step 3: Get OAuth access token
 * base_url: e.g. "http://www.flickr.com/services/oauth/access_token";
 */
OAuth.get_access_token = function(base_url, callback){
  
  var key = this.oauth_consumer_secret + "&" + this.oauth_request_token_secret;
  var oauth_nonce = this._createNonce(32);
  var oauth_timestamp = Math.round(new Date().getTime()/1000);

  var query_str = "";
  query_str += "oauth_consumer_key="+this.oauth_consumer_key;
  query_str += "&oauth_nonce="+oauth_nonce;
  query_str += "&oauth_signature_method="+this.oauth_signature_method;
  query_str += "&oauth_timestamp="+oauth_timestamp;
  query_str += "&oauth_token="+this.oauth_request_token;
  query_str += "&oauth_verifier="+this.oauth_verifier;
  query_str += "&oauth_version=1.0";

  var data = "GET" + "&" + encodeURIComponent(base_url) + "&" + encodeURIComponent(query_str);
  var sig = b64_hmac_sha1(key, data);
  var url = base_url + "?" + query_str + "&oauth_signature="+encodeURIComponent(sig);

  this._httpGet(url, function(txt, err){
    if (err!="OK"){
      callback(false, "Network error ("+ err+")");
    }else{
      res = OAuth._parseQueryString(txt);
      if (res.oauth_token && res.oauth_token_secret){
        OAuth.oauth_access_token = res.oauth_token;
        OAuth.oauth_access_token_secret = res.oauth_token_secret;
        callback(true);
      }else{
        callback(false, "Error getting access token!", res);
      }
    }
  });

};

/**
 * OAuth.call_api
 *
 * base_url: e.g. "http://api.flickr.com/services/rest";
 * param_obj: e.g. {
 *   format: "json",
 *   method: "flickr.test.login",
 *   nojsoncallback: 1
 * }
 *
 */
OAuth.call_api = function(base_url, param_obj, callback){
  
  var key = this.oauth_consumer_secret + "&" + this.oauth_access_token_secret;
  var oauth_nonce = this._createNonce(32);
  var oauth_timestamp = Math.round(new Date().getTime()/1000);
    
  param_obj.oauth_consumer_key = this.oauth_consumer_key;
  param_obj.oauth_nonce = oauth_nonce;
  param_obj.oauth_signature_method = this.oauth_signature_method;
  param_obj.oauth_timestamp = oauth_timestamp;
  param_obj.oauth_token = this.oauth_access_token;
  param_obj.oauth_version = "1.0";

  // sort props
  var props = [];
  for (var p in param_obj) props.push(p);
  props.sort();
  
  // create query string
  var query_str = props[0]+"="+param_obj[props[0]];
  for (var i=1;i<props.length;i++) query_str += "&"+props[i]+"="+param_obj[props[i]];
  
  var data = "GET" + "&" + encodeURIComponent(base_url) + "&" + encodeURIComponent(query_str);
  var sig = b64_hmac_sha1(key, data);
  var url = base_url + "?" + query_str + "&oauth_signature="+encodeURIComponent(sig);

  this._httpGet(url, function(txt, err){
    if (err!="OK"){
      callback(false, "Network error ("+ err+")");
    }else{
      callback(true, txt);
    }
  });

};

/**
 * OAuth._createNonce
 *
 * Creates a 'nonce' random string of specified length
 */
OAuth._createNonce = function(length) {
  var chars = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXTZ";
  var result = "";
  for (var i = 0; i < length; ++i) {
    var rnum = Math.floor(Math.random() * chars.length);
    result += chars.substring(rnum, rnum+1);
  }
  return result;
};

/**
 * OAuth._parseQueryString
 *
 * Parses result into an object
 */
OAuth._parseQueryString = function(qs) {

  // bug fix for d10: netLingo might return part of HTTP header!
  var p = qs.lastIndexOf("\r");  
  if (p>-1) qs = qs.substr(p+1);
  
  var res = {};
  qs.replace(
    new RegExp("([^?=&]+)(=([^&]*))?","g"),
    function($0,$1,$2,$3){res[$1]=$3;}
  );
  return res;
};

/**
 * OAuth._httpGet
 *
 * HTTP GET Request
 */
OAuth._httpGet = function(url, callback, vars){
  var n = {};
  if (!vars) vars = "";
  n.pNetID = getNetText(url, vars);
  n.callback = callback;
  n.stepFrame = function(){
    if (netDone(this.pNetID)){
      var cnt = _movie.actorList.count;
      for (var i=1;i<=cnt;i++){
        if (_movie.actorList[i]==this){
          _movie.actorList.deleteAt(i);
          break;
        }
      }
      if (this.callback) this.callback(netTextResult(this.pNetID), OAuth._errStr(netError(this.pNetID)));
      else trace(netTextResult(this.pNetID), OAuth._errStr(netError(this.pNetID)));
    }
  };
  _movie.actorList.add(n);
};

OAuth._errStr = function(err){
  switch(err){
    case "OK" : return "OK";
    case -128 : return "User cancelled.";
    case 0    : return "Everything is OK, peachy, and generally acceptable.";
    case 1    : return "Memory error.";
    case 4    : return "Bad MOA Class. Network Xtras may be improperly installed.";
    case 5    : return "Bad MOA Interface. Network Xtras may be improperly installed.";
    case 6    : return "Bad URL.";
    case 20   : return "Internal error. Browser detected a network or internal error.";
    case 42   : return "Response Header Field too long to be handled by NetLingo.";
    case 900  : return "File to be written to is read only.";
    case 903  : return "Disk is full.";
    case 905  : return "Bad URL.";
    case 2018 : return "postNetText fail.";
    case 4146 : return "Connection could not be established with the remote host.";
    case 4149 : return "Data supplied by the server was in an unexpected format.";
    case 4150 : return "Unexpected early closing of connection.";
    case 4152 : return "Data returned is truncated.";
    case 4154 : return "Operation could not be completed due to timeout.";
    case 4155 : return "Not enough memory available to complete the transaction.";
    case 4156 : return "Protocol reply to request indicates an error in the reply.";
    case 4157 : return "Transaction failed to be authenticated.";
    case 4159 : return "Invalid URL.";
    case 4164 : return "Could not create a socket.";
    case 4165 : return "Requested Object could not be found (URL may be incorrect).";
    case 4166 : return "Generic proxy failure.";
    case 4167 : return "Transfer was intentionally interrupted by client.";
    case 4168 : return "Failed network operation.";
    case 4240 : return "The network xtras weren't initialized properly.";
    case 4242 : return "Download stopped by netAbort(url).";
    case 4836 : return "Download stopped for an unknown reason.";
    default :
      if (err>4143 && err<4164)
        return "Failed network operation: " + err;
      else
        return "Unknown network error: " + err;
  }
}
